import copy
from typing import List

from PyQt5.QtCore import QObject, QCoreApplication, QEvent
from PyQt5.QtCore import pyqtSignal, QThread
from PyQt5.QtWidgets import QListView

from astra_installer.settings.component import Component
from astra_installer.settings.components import ComponentsSetting
from astra_installer.settings.manager import SettingsManager
from astra_installer_qt.components_manager import ComponentsManager
from astra_installer_qt.exclusive_components_manager import ExclusiveComponentsManager

translate = QCoreApplication.translate


class ComponentsLoader(QThread):
    ready = pyqtSignal()

    def __init__(self, csetting: ComponentsSetting) -> None:
        super().__init__()
        self.csetting = csetting

    def run(self) -> None:
        self.csetting.load_components()
        self.ready.emit()


class ComponentsDialog(QObject):
    def __init__(self, widget: QListView, backend: ComponentsSetting) -> None:
        super().__init__()
        self.loader = ComponentsLoader(backend)
        self.widget = widget
        self.components_manager = self.create_manager(backend)
        self.components = []  # type: List[Component]
        self.setup()

    def create_manager(self, backend) -> ComponentsManager:
        if backend.exclusive_components:
            return ExclusiveComponentsManager(self.widget, backend)
        else:
            return ComponentsManager(self.widget, backend)

    def setup(self) -> None:
        if SettingsManager.systeminfo.oem:
            self.loader.ready.connect(self.components_manager.setup_components_and_update_label)
            self.loader.start()

    def setup_backend(self) -> None:
        self.components_manager.setup_backend()

    def show(self) -> None:
        active_window = QCoreApplication.instance().activeWindow()
        self.components = copy.deepcopy(self.components_manager.model.components)
        super().exec()
        if active_window:
            active_window.activateWindow()

    def apply(self) -> None:
        self.components = copy.deepcopy(self.components_manager.model.components)
        # self.components_manager.indicate_loading()

    def reset(self) -> None:
        self.components_manager.model.components = copy.deepcopy(self.components)
        # self.components_manager.indicate_loading()

    def retranslate(self, locale: str) -> None:
        self.components_manager.retranslate()

    # noinspection PyPep8Naming
    def changeEvent(self, event: QEvent) -> None:
        if event.type() == QEvent.LanguageChange:
            self.retranslateUi(self)
