from PyQt5.QtCore import QObject, Qt, QEvent
from PyQt5.QtGui import QFontMetrics
from PyQt5.QtWidgets import QLabel


class ElidedTextLabel(QObject):
    def __init__(self, label: QLabel, text: str = '') -> None:
        super(ElidedTextLabel, self).__init__()
        self.label = label
        self.__text = text
        self.metrics = QFontMetrics(label.font())
        self.label.installEventFilter(self)

    @property
    def text(self) -> str:
        return self.__text

    @text.setter
    def text(self, value: str) -> None:
        self.__text = value
        self.update_elided_text()

    def update_elided_text(self) -> None:
        elided_text = self.metrics.elidedText(self.text, Qt.ElideRight, self.label.width())
        self.label.setText(elided_text)

    # noinspection PyPep8Naming
    def eventFilter(self, qobject: QObject, event: QEvent) -> bool:
        if event.type() == QEvent.Resize:
            self.update_elided_text()
            return True
        else:
            return False
