from collections import OrderedDict
from typing import TYPE_CHECKING

from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtWidgets import QLabel

if TYPE_CHECKING:
    from gettext import gettext as _


class ErrorManager(QObject):
    error = pyqtSignal()
    noerror = pyqtSignal()

    def __init__(self, label_icon: QLabel, label_text: QLabel, mainwindow) -> None:
        super(ErrorManager, self).__init__()
        self.current_text = ''
        self.label_icon = label_icon
        self.label_text = label_text
        self.mw = mainwindow
        self.errors = OrderedDict()  # type: OrderedDict[str, str]
        self.setup()

    def setup(self) -> None:
        self.hide()

    def hide(self) -> None:
        self.label_text.hide()
        self.label_icon.hide()
        self.noerror.emit()

    def show(self, id: str, text: str) -> None:
        if id == 'password' and not (
                len(self.mw.lineEditPassword.text()) or len(self.mw.lineEditPasswordConfirmation.text())):
            pass
            # self.label_text.hide()
            # self.label_icon.hide()
        else:
            self.current_text = text
            self.label_text.setText(_(text))
            self.label_text.show()
            self.label_icon.show()
        self.error.emit()

    def show_error(self, id: str, text: str) -> None:
        if id in self.errors:
            del self.errors[id]
            self.errors[id] = text
        else:
            self.errors[id] = text
        self.show(id, text)

    def hide_error(self, id: str) -> None:
        if id in self.errors:
            del self.errors[id]
            if len(self.errors) > 0:
                id, text = self.errors.popitem()
                self.errors[id] = text
                self.show(id, text)
            else:
                self.hide()

    def retranslate(self, locale: str) -> None:
        self.label_text.setText(_(self.current_text))
