from typing import TYPE_CHECKING

from PyQt5.QtCore import QObject, QCoreApplication
from PyQt5.QtWidgets import QPushButton, QLabel, QMainWindow

from astra_installer.settings.manager import SettingsManager
from astra_installer.settings.systeminfo import AstraEdition
from astra_installer_qt.grub.ui.grub_settings_dialog import GrubSettingsDialog

translate = QCoreApplication.translate

if TYPE_CHECKING:
    from gettext import gettext as _


class GrubManager(QObject):
    def __init__(self, main_window: QMainWindow, settings_button: QPushButton,
                 info_label: QLabel, title_label: QLabel) -> None:
        super().__init__()
        self.__title_label = title_label
        self.__settings_button = settings_button
        self.__info_label = info_label
        self.__settings_dialog = GrubSettingsDialog(parent=main_window)
        self.__setup_grub_password = False  # type: bool
        self.__grub_password = ''  # type : str
        self.__grub_password_confirmation = ''  # type : str
        self.__setup()

    @property
    def setup_grub_password(self) -> bool:
        return self.__setup_grub_password

    @property
    def grub_password(self) -> str:
        return self.__grub_password

    @property
    def grub_password_confirmation(self) -> str:
        return self.__grub_password_confirmation

    def retranslate(self, locale: str) -> None:
        self.__settings_dialog.retranslate(locale)
        self.__update_info_label()

    def __setup(self) -> None:
        if SettingsManager.systeminfo.arch == "e2k-8c":
            self.__title_label.setVisible(False)
            self.__settings_button.setVisible(False)
            self.__info_label.setVisible(False)
            return

        self.__setup_grub_password = (SettingsManager.systeminfo.edition == AstraEdition.SE)
        self.__settings_button.clicked.connect(self.__settings_button_clicked)
        self.__settings_dialog.settings_changed.connect(self.__settings_changed)
        self.__update_info_label()

    def __settings_button_clicked(self) -> None:
        self.__settings_dialog.show()

    def __settings_changed(self,
                           setup_grub_password: bool = False,
                           grub_password: str = '',
                           grub_password_confirmation: str = '') -> None:
        self.__setup_grub_password = setup_grub_password
        self.__grub_password = grub_password
        self.__grub_password_confirmation = grub_password_confirmation
        self.__update_info_label()

    def __update_info_label(self) -> None:
        if not self.__setup_grub_password:
            text = self.tr('Password not setuped')
        else:
            if self.__grub_password:
                text = self.tr('Password setuped')
            else:
                text = self.tr('Setup the administrator password')
        self.__info_label.setText(text)
