from collections import deque
from typing import List

from PyQt5.QtCore import QObject, QTimer
from PyQt5.QtWidgets import QLabel

from astra_installer_qt.pixmap_label import PixmapLabel


class ImagePresentationManager(QObject):
    def __init__(self, files: List[str], label: QLabel) -> None:
        super(ImagePresentationManager, self).__init__()
        self.files = files
        self.pixmap_label = PixmapLabel(label)
        pixmaps = PixmapLabel.gen_pixmaps(self.files)
        self.pixmaps = deque(pixmaps)
        self.timer = QTimer(self)

    def run(self) -> None:
        self.next_image()
        self.timer.timeout.connect(self.next_image)
        self.timer.start(30000)

    def next_image(self) -> None:
        if len(self.pixmaps) > 0:
            pixmap = self.pixmaps.popleft()
            self.pixmap_label.pixmap = pixmap
            self.pixmaps.append(pixmap)
