from typing import TYPE_CHECKING

from PyQt5.QtCore import QObject, QCoreApplication
from PyQt5.QtWidgets import QPushButton, QLabel, QMainWindow

from astra_installer.settings.manager import SettingsManager
from astra_installer_qt.kernel.ui.kernel_settings_dialog import KernelSettingsDialog

translate = QCoreApplication.translate

if TYPE_CHECKING:
    from gettext import gettext as _

class KernelManager(QObject):
    def __init__(self, main_window: QMainWindow,
                 settings_button: QPushButton,
                 title_label: QLabel,
                 info_label: QLabel) -> None:
        super().__init__()
        self.backend = SettingsManager.kernel
        self.__settings_button = settings_button
        self.__title_label = title_label
        self.__info_label = info_label
        self.__settings_dialog = KernelSettingsDialog(parent=main_window)
        self.__setup()

    def retranslate(self, locale: str) -> None:
        self.__settings_dialog.retranslate(locale)
        self.__update_info_label()

    def __setup(self) -> None:
        if self.backend.package == "linux-image-*-generic":
            self.__settings_button.setVisible(False)
            self.__title_label.setVisible(False)
            self.__info_label.setVisible(False)
            return
        self.__settings_button.clicked.connect(self.__settings_button_clicked)
        self.__settings_dialog.settings_changed.connect(self.__settings_changed)
        self.__update_info_label()

    def __settings_button_clicked(self) -> None:
        self.__settings_dialog.show()

    def __settings_changed(self, kernel_package: str = '') -> None:
        if kernel_package:
            self.backend.package = kernel_package
        self.__update_info_label()

    def __update_info_label(self) -> None:
        text = self.tr("The kernel {} will be installed") \
            .format(self.backend.package)
        self.__info_label.setText(text)
