from typing import Optional, TYPE_CHECKING

from PyQt5 import uic
from PyQt5.QtCore import Qt, QCoreApplication, pyqtSignal
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QListWidget
from PyQt5.QtWidgets import QWidget, QDialog, QPushButton

from astra_installer.settings.manager import SettingsManager
from astra_installer_qt.utils import Utils as QtUtils

if TYPE_CHECKING:
    from gettext import gettext as _

translate = QCoreApplication.translate

dialog_ui = QtUtils.file_path('kernel', 'ui', 'kernel_settings_dialog.ui')
UiFormClass, UiFormBase = uic.loadUiType(dialog_ui)  # type: QDialog, QDialog


class KernelSettingsDialog(UiFormClass, UiFormBase):
    settings_changed = pyqtSignal(str)

    def __init__(self, parent: QWidget = None) -> None:
        super().__init__(parent=parent)
        self.backend = SettingsManager.kernel
        self._kernel_list_widget = None  # type: Optional[QListWidget]
        self._apply_push_button = None # type: Optional[QPushButton]
        self._cancel_push_button = None # type: Optional[QPushButton]
        self.setupUi(self)
        self.__setup()

    def __update_kernels(self):
        if self.backend.package == 'linux-image-*-generic':
            return

        self._kernel_list_widget.addItems(self.backend.available_packages)
        if self._kernel_list_widget.count() > 0:
            item = self._kernel_list_widget.findItems(self.backend.package, Qt.MatchExactly)[-1]
            self._kernel_list_widget.setCurrentItem(item)

    def __setup(self) -> None:
        self.setModal(True)
        self.adjustSize()
        self.__update_kernels()
        self._apply_push_button.clicked.connect(self.accept)
        self._cancel_push_button.clicked.connect(self.reject)

    def show(self) -> None:
        active_window = QCoreApplication.instance().activeWindow()
        self.setVisible(True)
        self.exec()
        if active_window:
            active_window.activateWindow()

    def retranslate(self, __: str) -> None:
        self.retranslateUi(self)

    def accept(self) -> None:
        self.settings_changed.emit(self._kernel_list_widget.currentItem().text())
        return super().accept()

    def reject(self) -> None:
        item = self._kernel_list_widget.findItems(self.backend.package, Qt.MatchExactly)[-1]
        self._kernel_list_widget.setCurrentItem(item)
        self.settings_changed.emit(self._kernel_list_widget.currentItem().text())
        return super().reject()

    def closeEvent(self, event: QCloseEvent) -> None:
        event.ignore()
        self.reject()
