from typing import TYPE_CHECKING

from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QComboBox

from astra_installer.settings.manager import SettingsManager

if TYPE_CHECKING:
    from gettext import gettext as _


class LayoutSwitchHotkeyManager(QObject):
    def __init__(self, combobox: QComboBox) -> None:
        super(LayoutSwitchHotkeyManager, self).__init__()
        self.combobox = combobox
        self.selected_index = 0
        self.setup()

    def setup(self) -> None:
        self.combobox.activated.connect(self.on_combobox_activated)
        self.add_keys()

    def add_keys(self) -> None:
        self.combobox.clear()
        key_to_select = next(iter(SettingsManager.layout.selected_keys), SettingsManager.layout.default_key)
        for index, pair in enumerate(SettingsManager.layout.keys.items()):
            key, name = pair
            self.combobox.addItem(_(name), key)
            if key == key_to_select:
                self.selected_index = index
        self.combobox.setCurrentIndex(self.selected_index)
        if SettingsManager.systeminfo.arch == "e2k-8c":
            self.combobox.model().sort(1)

    def on_combobox_activated(self) -> None:
        self.selected_index = self.combobox.currentIndex()
        self.setup_backend()
        SettingsManager.layout.apply()

    def setup_backend(self) -> None:
        SettingsManager.layout.selected_keys = [self.combobox.currentData()]

    def retranslate(self) -> None:
        self.add_keys()
