from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QApplication, QPushButton, QStackedWidget, QTextEdit, QLabel

from astra_installer.settings.manager import SettingsManager

translate = QApplication.translate


class LogManager(QObject):
    def __init__(self, button: QPushButton,
                 stacked_widget: QStackedWidget,
                 log_text_edit: QTextEdit,
                 info_label: QLabel,
                 image_label: QLabel) -> None:
        super(LogManager, self).__init__()
        self.button = button
        self.stkw = stacked_widget
        self.log_text_edit = log_text_edit
        self.info_label = info_label
        self.image_label = image_label
        self.pages_data = {
            'slides': dict(index=0,
                           text=self.tr('Open log'),
                           next_page='log',
                           callback=self.on_presentation_opened),
            'log': dict(index=1,
                        text=self.tr('Close log'),
                        next_page='slides',
                        callback=self.on_log_opened),
            'info': dict(index=2,
                         text=self.tr('Open log'),
                         next_page='log',
                         callback=self.on_info_opened)
        }
        self.current_page_data = self.pages_data['slides']
        self.setup()

    def setup(self) -> None:
        self.button.clicked.connect(self.button_clicked)
        self.switch_pages('slides')

    def button_clicked(self) -> None:
        self.image_label.setHidden(False)
        self.log_text_edit.setHidden(False)
        self.switch_pages(self.current_page_data['next_page'])

    def switch_pages(self, next_page: str) -> None:
        self.current_page_data = self.pages_data[next_page]
        self.current_page_data['callback']()
        self.stkw.setCurrentIndex(self.current_page_data['index'])
        self.button.setText(self.current_page_data['text'])

    def log(self, text: str) -> None:
        self.log_text_edit.appendPlainText(text)

    def on_presentation_opened(self):
        self.image_label.setHidden(True)
        self.log_text_edit.setHidden(True)

    def on_info_opened(self):
        self.image_label.setHidden(True)
        self.log_text_edit.setHidden(True)

    def on_log_opened(self):
        self.image_label.setHidden(False)
        self.log_text_edit.setHidden(False)

    def open_installation_info(self, time: str, info: str = None) -> None:
        log_data = self.pages_data['log']
        log_data['next_page'] = 'info'
        log_data['text'] = self.tr('Close log')
        self.pages_data['log'] = log_data
        if info:
            text = translate('text', "<html><head/><body>Installation time: {time}<br>{info}</body></html>")
            self.info_label.setText(text.format(version=SettingsManager.systeminfo.release, time=time, info=info))
        else:
            text = translate('text', "<html><head/><body>Installation time: {time}</body></html>")
            self.info_label.setText(text.format(version=SettingsManager.systeminfo.release, time=time))
        self.switch_pages('info')

    def open_installation_log(self) -> None:
        self.switch_pages('log')

    def retranslate(self, locale: str) -> None:
        self.pages_data['slides']['text'] = self.tr('Open log')
        self.pages_data['info']['text'] = self.tr('Open log')
        self.pages_data['log']['text'] = self.tr('Close log')
        self.button.setText(self.current_page_data['text'])
