from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtWidgets import QComboBox, QLabel, QApplication

from astra_installer.settings.manager import SettingsManager
from astra_installer_qt.components_manager import ComponentsManager

translate = QApplication.translate


class ModeManager(QObject):
    locale_changed = pyqtSignal()

    def __init__(self, combobox: QComboBox,
                 label: QLabel,
                 cmanager: ComponentsManager) -> None:
        super().__init__()
        self.combobox = combobox
        self.label = label
        self.cmanager = cmanager
        self.selected_index = 0
        self.__maximum_mode_index = 0
        self.setup()

    def setup(self) -> None:
        self.combobox.activated.connect(self.apply)
        self.add_modes()
        self.check_release()

    def add_modes(self) -> None:
        self.combobox.clear()
        self.selected_index = 0
        for index, mode in enumerate(SettingsManager.mode.modes):
            self.combobox.addItem(translate('mode', mode.name), mode)
            if mode.selected:
                self.selected_index = index
            if mode.name == "Maximum":
                self.__maximum_mode_index = index
        self.combobox.setCurrentIndex(self.selected_index)

    def apply(self) -> None:
        selected_mode = self.combobox.currentData()
        for mode in SettingsManager.mode.modes:
            mode.selected = True if mode.ident == selected_mode.ident else False
        self.cmanager.update_components(selected_mode.features)

    def check_release(self) -> None:
        if SettingsManager.systeminfo.arch == "e2k-8c":
            self.combobox.setCurrentIndex(self.__maximum_mode_index)
            self.label.hide()
            self.combobox.hide()

    def retranslate(self, locale: str) -> None:
        self.add_modes()
        self.check_release()
        self.apply()
