from typing import Optional

from PyQt5.QtGui import QWindow, QPalette
from PyQt5.QtWidgets import QWidget, QGroupBox, QVBoxLayout, QSizePolicy


class BlivetGuiWidget(QGroupBox):
    def __init__(self, settings_dialog, parent: QWidget = None) -> None:
        super().__init__(parent=parent)
        self.__settings_dialog = settings_dialog
        self.__blivet_gui = None  # type: Optional[InstallerBlivetGUI]
        self.__blivet_widget = None  # type: Optional[QWidget]
        self.__setup()

    @property
    def blivet_gui(self):
        return self.__blivet_gui

    def retranslate(self, _locale: str) -> None:
        self.__init_blivet_gui()

    def __setup(self) -> None:
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(layout)

    def __init_blivet_gui(self) -> None:
        if self.__blivet_widget:
            self.layout().removeWidget(self.__blivet_widget)

        from astra_installer_qt.storage.blivet.installer_blivet_gui import InstallerBlivetGUI  # TODO
        if not self.__blivet_gui:
            self.__blivet_gui = InstallerBlivetGUI(settings_dialog=self.__settings_dialog)
            self.__blivet_gui.main_window.connect('size-allocate', self.__on_blivet_gui_size_allocate)
            self.__blivet_gui.main_window.connect('set-focus', self.__on_blivet_gui_set_focus)
            self.__blivet_gui.set_background_color(self.palette().color(QPalette.Base).name())

        window = QWindow.fromWinId(self.__blivet_gui.main_window.get_window().get_xid())
        self.__blivet_widget = QWidget.createWindowContainer(window)
        self.__blivet_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.__blivet_widget.setMinimumSize(0, 365)

        self.layout().addWidget(self.__blivet_widget)

    def __on_blivet_gui_size_allocate(self, widget, allocation) -> None:
        if widget == self.__blivet_gui.main_window:
            self.__blivet_widget.resize(allocation.width, allocation.height)
            self.__blivet_widget.updateGeometry()

    def __on_blivet_gui_set_focus(self, _window, _widget) -> None:
        self.parent().setFocus()
