import blivet.formats

from typing import Optional

from blivetgui.blivet_utils import BlivetUtils
from blivetgui.logs import set_logging

from astra_installer_kickstart.installer_storage import InstallerStorage


class InstallerBlivetUtils(BlivetUtils):
    installer_mode = True

    def __init__(self) -> None:
        self.ignored_disks = None
        self.exclusive_disks = None
        self._resizable_filesystems = None
        self.storage = None  # type: Optional[InstallerStorage]
        _log_file, self.log = set_logging(component="blivet-gui-utils")
        blivet.formats.fs.NTFS._formattable = True
        blivet.formats.fs.NTFS._supported = True

    def get_disks(self):
        if not self.storage:
            return []

        return super().get_disks()

    def get_group_devices(self):
        if not self.storage:
            devices = dict()
            devices["lvm"] = None
            devices["raid"] = None
            devices["btrfs"] = None
            return devices

        return super().get_group_devices()

    def device_resizable(self, blivet_device):
        self.installer_mode = False
        try:
            return super().device_resizable(blivet_device)
        finally:
            self.installer_mode = True

    def get_supported_filesystems(self, installer_mode=False, forced_biosboot=False):
        _fs_types = []

        additional_fs = ["swap", "lvmpv"]
        if installer_mode:
            additional_fs.extend(["biosboot", "prepboot"])

        for cls in blivet.formats.device_formats.values():
            obj = cls()

            supported_fs = (obj.type not in ("tmpfs",) and
                            (obj.supported or (obj.type in ("biosboot",) and forced_biosboot)) and
                            obj.formattable and
                            (isinstance(obj, blivet.formats.fs.FS) or
                             obj.type in additional_fs))
            if supported_fs:
                _fs_types.append(obj)

        return sorted(_fs_types, key=lambda fs: fs.type)
