from typing import TYPE_CHECKING

import gi
gi.require_version("Gtk", "3.0")
gi.require_version("Pango", "1.0")

from gi.repository import Gtk, Pango

from blivetgui.dialogs.device_info_dialog import DeviceInformationDialog

if TYPE_CHECKING:
    from gettext import gettext as _

class InstallerDiskInformationDialog(DeviceInformationDialog):
    def add_device_info(self):
        """ Display information about the device
        """

        # device name and type header
        device_type_label = Gtk.Label()
        self.grid.attach(child=device_type_label, left=0, top=0, width=2, height=1)

        # unknow device type
        if self.device.type not in self.type_dict.keys():
            if self.device.type != "disk":
                info = _("Unknown device {name}").format(name=self.device.name)
            else:
                info = _("Disk {name}").format(name=self.device.name)
        else:
            info = "{type} {name}".format(type=self.type_dict[self.device.type][0],
                                          name=self.device.name)
        device_type_label.set_markup("<b>%s</b>" % info)

        # device info header
        info_type_label = Gtk.Label(label="<i>%s</i>" % _("Basic information"), use_markup=True)
        self.grid.attach(child=info_type_label, left=0, top=1, width=2, height=1)
        info_type_label.set_xalign(0)
        info_type_label.set_yalign(0)

        device_info_label = Gtk.Label()
        device_info_label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
        self.grid.attach(device_info_label, left=0, top=2, width=1, height=1)

        # 'basic' information about selected device
        info = _(" • <i>Name:</i> {name}\n").format(name=self.device.name)
        info += _(" • <i>Path:</i> {path}\n").format(path=self.device.path)
        info += _(" • <i>Size:</i> {size}\n").format(size=str(self.device.size))

        device_info_label.set_markup(info)
        device_info_label.set_xalign(0)
        device_info_label.set_yalign(0)

        # 'advanced' information about selected device (specific for each device type)
        if self.device.type in self.type_dict.keys() and self.type_dict[self.device.type][1]:
            adv_info_label = Gtk.Label()
            self.grid.attach(adv_info_label, left=1, top=2, width=1, height=1)

            adv_info_fn = self.type_dict[self.device.type][1]
            adv_info_label.set_markup(adv_info_fn())
            adv_info_label.set_xalign(0)
            adv_info_label.set_yalign(0)

    def add_format_info(self):
        """ Display information about format of the device
        """

        # device format header
        info_type_label = Gtk.Label(label="<i>%s</i>" % _("Device format"), use_markup=True)
        self.grid.attach(child=info_type_label, left=0, top=3, width=2, height=1)
        info_type_label.set_xalign(0)
        info_type_label.set_yalign(0)

        # information about device format
        fmt_info_label = Gtk.Label()
        fmt_info_label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
        self.grid.attach(fmt_info_label, left=0, top=4, width=2, height=1)

        if self.device.format and self.device.format.type:
            info = ""
            if self.device.format.type != "disklabel":
                info += _(" • <i>Type:</i> {type}\n").format(type=self.device.format.type)
            info += _(" • <i>UUID:</i> {uuid}\n").format(uuid=self.device.format.uuid)
            if hasattr(self.device.format, "label_type") and self.device.format.label_type:
                info += _(" • <i>Partition table:</i> {label}\n").format(label=self.device.format.label_type)
        else:
            info = _(" • <i>Type:</i> None")

        fmt_info_label.set_markup(info)
        fmt_info_label.set_xalign(0)
        fmt_info_label.set_yalign(0)
