#
# Copyright (C) 2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vojtech Trefny <vtrefny@redhat.com>
#

from typing import TYPE_CHECKING

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

from blivetgui.main_menu import MainMenu

if TYPE_CHECKING:
    from gettext import gettext as _


class InstallerMainMenu(MainMenu):
    def __init__(self, blivet_gui) -> None:
        super().__init__(blivet_gui)
        self.__main_menu = blivet_gui.builder.get_object("main_menu")
        self.__menu_item_reload = blivet_gui.builder.get_object("menuitem_reload")
        self.__menu_item_actions = blivet_gui.builder.get_object("menuitem_actions")
        self.__menu_item_quit = blivet_gui.builder.get_object("menuitem_quit")
        self.__menu_item_about = blivet_gui.builder.get_object("menuitem_about")
        self.__menuitem_apply = Gtk.MenuItem(name="menuitem_apply",
                                             label=_('Write configuration on disk'))
        self.__setup()

    def __setup(self) -> None:
        self.__menu_item_about.hide()
        self.__menu_item_quit.hide()
        self.__menu_item_reload.set_label(_('Reload layout configuration'))
        self.__menuitem_apply.connect('activate', self.__menuitem_apply_activate)
        self.__main_menu.append(self.__menuitem_apply)
        self.__menuitem_apply.show()

    def __menuitem_apply_activate(self, _item) -> None:
        self.blivet_gui.apply_event()
        self.blivet_gui.set_formatted_manually()
