from typing import Optional

from PyQt5 import uic
from PyQt5.QtCore import QCoreApplication
from PyQt5.QtWidgets import QWidget, QDialog, QLineEdit, QPushButton, QLabel

from astra_installer_qt.utils import Utils as QtUtils

translate = QCoreApplication.translate

dialog_ui = QtUtils.file_path('storage', 'ui', 'passphrase_dialog.ui')
UiFormClass, UiFormBase = uic.loadUiType(dialog_ui)  # type: QDialog, QDialog


class PassphraseDialog(UiFormClass, UiFormBase):
    def __init__(self, parent: QWidget = None) -> None:
        self._passphrase_line_edit = None  # type: Optional[QLineEdit]
        self._confirm_passphrase_line_edit = None  # type: Optional[QLineEdit]
        self._ok_push_button = None  # type: Optional[QPushButton]
        self._cancel_push_button = None  # type: Optional[QPushButton]
        self._error_label = None  # type: Optional[QLabel]
        super().__init__(parent=parent)
        self.setupUi(self)
        self.__setup()

    @property
    def passphrases(self) -> str:
        return self._passphrase_line_edit.text()

    def __setup(self) -> None:
        self._cancel_push_button.clicked.connect(self.reject)
        self._ok_push_button.clicked.connect(self.accept)
        self._passphrase_line_edit.textEdited.connect(self.__passphrase_line_edit_text_edited)
        self._confirm_passphrase_line_edit.textEdited.connect(self.__confirm_passphrase_line_edit_text_edited)
        self._ok_push_button.setEnabled(False)

    def __passphrase_line_edit_text_edited(self, text: str) -> None:
        self.__validate()

    def __confirm_passphrase_line_edit_text_edited(self) -> None:
        self.__validate()

    def __validate(self) -> None:
        if not self.passphrases:
            self.__show_error(translate('text', 'passphrase is empty'))
            return

        if len(self.passphrases) < 8:
            self.__show_error(translate('text', 'the passphrase is too short'))
            return

        confirm_passphrase = self._confirm_passphrase_line_edit.text()
        if not confirm_passphrase:
            self.__show_error(translate('text', 'confirm passphrase'))
            return

        if self.passphrases != confirm_passphrase:
            self.__show_error(translate('text', 'passphrases do not match'))
            return

        self.__clear_error()

    def __show_error(self, text: str) -> None:
        self._error_label.setText(text)
        self._ok_push_button.setEnabled(False)

    def __clear_error(self) -> None:
        self._error_label.clear()
        self._ok_push_button.setEnabled(True)
