import os
import subprocess
from subprocess import call
from threading import Thread
from typing import Callable

from PyQt5.QtCore import QEventLoop
from PyQt5.QtWidgets import QApplication, QMessageBox, QWidget

from astra_installer.utils import Utils as BackendUtils

translate = QApplication.translate


class Utils:
    @classmethod
    def file_path(cls, *paths) -> str:
        package_directory = os.path.dirname(os.path.abspath(__file__))
        return os.path.join(package_directory, *paths)

    @staticmethod
    def is_wm_present() -> bool:
        # Эта реализация работает только при запуске с su-to-root:
        # os.getenv('DESKTOP_SESSION') and os.getenv('XDG_CURRENT_DESKTOP')
        # В PyQt5.QtX11Extras.QX11Info отсутствует isCompositingManagerRunning:
        # type object 'QX11Info' has no attribute 'isCompositingManagerRunning'
        # Значит, просто поищем процесс fly-wm
        try:
            uid = BackendUtils.get_sudo_uid()
            subprocess.check_call("ps -u {uid} | grep fly-wm".format(uid=uid),
                                  stdout=subprocess.PIPE, stderr=subprocess.DEVNULL,
                                  shell=True, universal_newlines=True)
            return True
        except Exception:
            return False

    @staticmethod
    def reboot() -> None:
        call(['systemctl', 'reboot'])

    @staticmethod
    def poweroff() -> None:
        call(['systemctl', 'poweroff'])

    @staticmethod
    def check_root_user() -> bool:
        if os.getuid() != 0:
            QMessageBox.critical(None, translate('text', 'Unprivileged user'),
                                 translate('text', 'You run the program under an unprivileged user.\n'
                                                   'Please run the program under the root user.'))
            return False
        else:
            return True

    @staticmethod
    def wait_thread(widget: QWidget, method: Callable, args: list):
        event_loop = QEventLoop()
        values = []
        was_exit = [False]

        def do_it(return_values: list):
            return_values.append(method(*args))
            was_exit[0] = True
            event_loop.exit()

        thread = Thread(target=do_it, args=(values,))
        widget.show()
        thread.start()
        if not was_exit[0]:
            event_loop.exec()
        widget.close()

        return values[0]
